// Generated by dts-bundle-generator v9.5.1

/**
 * A Priority Queue implementation that is given numeric priorities and returns
 * whichever item has the highest (by default) or the lowest (if specified in
 * the constructor). Priorities must be numbers, but negative numbers and
 * fractions are allowed.
 *
 * The use of a fixed numerical priority is meant to add some speed to the
 * implementation as opposed to implementations that re-calculate priorities
 * with comparators.
 */
export declare class PriorityQueue<T> {
	readonly isLowestToHighest: boolean;
	/**
	 * @param isLowestToHighest If true, {@link poll} will return the item with
	 * the lowest given value for "priority," Useful if your priorities are
	 * actually costs. Defaults to false.
	 */
	constructor(isLowestToHighest?: boolean);
	/**
	 * The number of items in the queue
	 */
	length: number;
	/**
	 * A heap sorted from highest to lowest number. If {@link isLowestToHighest}
	 * is true, then we will multiply all given priorities by -1 before putting
	 * them in this array, thus making it so we always sort from highest to lowest
	 * internally.
	 */
	private _priorities;
	/**
	 * The heap datastructure that holds the actual data. The priority of each
	 * entry is stored in the {@link _priorities} array at the same index.
	 */
	private _data;
	/**
	 * Add an element at a given priority to the queue, runs in O(log n) time.
	 * @param priority The priority to add the element
	 * @param item the item itself
	 */
	add(priority: number, item: T): void;
	/**
	 * Returns the highest priority item without removing it from the queue, or
	 * `undefined` if there is no data.
	 */
	peek(): undefined | T;
	/**
	 * Returns the priority of the next item to be returned from the queue.
	 */
	peekPriority(): undefined | number;
	/**
	 * Returns (and removes) the highest priority item from the heap.
	 * Runs in O(log n) time.
	 */
	poll(): undefined | T;
}

export {};
